@@ -19,21 +19,31 @@ def test_version_from_git(wd):
     assert wd.version == '0.1.dev0'
 
     wd.commit_testfile()
-    assert wd.version.startswith('0.1.dev1+')
+    assert wd.version.startswith('0.1.dev1+g')
     assert not wd.version.endswith('1-')
 
     wd('git tag v0.1')
     assert wd.version == '0.1'
 
     wd.write('test.txt', 'test2')
-    assert wd.version.startswith('0.2.dev0+')
+    assert wd.version.startswith('0.2.dev0+g')
 
     wd.commit_testfile()
-    assert wd.version.startswith('0.2.dev1+')
+    assert wd.version.startswith('0.2.dev1+g')
 
     wd('git tag version-0.2')
     assert wd.version.startswith('0.2')
 
+    wd.commit_testfile()
+    wd('git tag version-0.2.post210+gbe48adfpost3+g0cc25f2')
+    assert wd.version.startswith('0.2')
+
+
+@pytest.mark.issue(179)
+def test_unicode_version_scheme(wd):
+    scheme = b'guess-next-dev'.decode('ascii')
+    assert wd.get_version(version_scheme=scheme)
+
 
 @pytest.mark.issue(108)
 @pytest.mark.issue(109)
@@ -92,7 +102,25 @@ def test_find_files_stop_at_root_git(wd):
     assert integration.find_files(str(wd.cwd/'project')) == []
 
 
+@pytest.mark.issue(128)
+def test_parse_no_worktree(tmpdir):
+    ret = git.parse(str(tmpdir))
+    assert ret is None
+
+
 def test_alphanumeric_tags_match(wd):
     wd.commit_testfile()
     wd('git tag newstyle-development-started')
-    assert wd.version.startswith('0.1.dev1+')
+    assert wd.version.startswith('0.1.dev1+g')
+
+
+def test_git_archive_export_ignore(wd):
+    wd.write('test1.txt', 'test')
+    wd.write('test2.txt', 'test')
+    wd.write('.git/info/attributes',
+             # Explicitly include test1.txt so that the test is not affected by
+             # a potentially global gitattributes file on the test machine.
+             '/test1.txt -export-ignore\n/test2.txt export-ignore')
+    wd('git add test1.txt test2.txt')
+    wd.commit()
+    assert integration.find_files(str(wd.cwd)) == ['test1.txt']
